print("import: 'palettable'")
import palettable

print("import: 'palettable.cmocean'")
import palettable.cmocean

print("import: 'palettable.colorbrewer'")
import palettable.colorbrewer

print("import: 'palettable.cubehelix'")
import palettable.cubehelix

print("import: 'palettable.matplotlib'")
import palettable.matplotlib

print("import: 'palettable.mycarta'")
import palettable.mycarta

print("import: 'palettable.tableau'")
import palettable.tableau

print("import: 'palettable.wesanderson'")
import palettable.wesanderson

